import sys
from copy import deepcopy
from datetime import datetime
from importlib import import_module
from unittest import skip
from zoneinfo import ZoneInfo

import time_machine
from bs4 import BeautifulSoup
from dateutil.relativedelta import relativedelta
from django.apps import apps as django_apps
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.test.utils import override_settings, tag
from django.urls.base import reverse
from django.urls.exceptions import NoReverseMatch
from django_extensions.management.color import color_style
from django_webtest import WebTest
from edc_adverse_event.constants import AE_ROLE, TMG_ROLE
from edc_appointment.constants import IN_PROGRESS_APPT, SCHEDULED_APPT
from edc_appointment.models import Appointment
from edc_auth.auth_updater import AuthUpdater
from edc_auth.constants import AUDITOR_ROLE, CLINICIAN_ROLE, STAFF_ROLE
from edc_auth.site_auths import site_auths
from edc_consent.site_consents import site_consents
from edc_constants.constants import YES
from edc_dashboard.url_names import url_names
from edc_export.constants import DATA_EXPORTER_ROLE
from edc_lab.auth_objects import LAB_TECHNICIAN_ROLE
from edc_randomization.admin import register_admin
from edc_randomization.site_randomizers import site_randomizers
from edc_sites.utils import add_or_update_django_sites
from edc_test_utils.webtest import login
from edc_utils import get_utcnow
from model_bakery import baker
from webtest.app import AppError

from meta_rando.randomizers import RandomizerPhaseThree
from meta_screening.models import ScreeningPartOne, ScreeningPartThree, ScreeningPartTwo
from meta_screening.models.subject_screening import SubjectScreening
from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_screening.tests.options import (
    get_part_one_eligible_options,
    get_part_three_eligible_options,
    get_part_two_eligible_options,
)
from meta_sites.sites import all_sites

style = color_style()
User = get_user_model()

app_prefix = "meta"
screening_listboard_url = f"{app_prefix}_dashboard:screening_listboard_url"


@override_settings(
    SIMPLE_HISTORY_PERMISSIONS_ENABLED=True,
    EDC_AUTH_SKIP_SITE_AUTHS=True,
    EDC_AUTH_SKIP_AUTH_UPDATER=False,
    SITE_ID=10,
)
class AdminSiteTest(MetaTestCaseMixin, WebTest):
    sid_count = 5

    menu_labels = [
        "Screening",
        "Subjects",
        "Specimens",
        "Adverse events",
        "TMG Reports",
        "Pharmacy",
        "Action items",
        "Export data",
        "Synchronization",
        "Switch sites",
        "Log out",
    ]

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        site_auths.initialize()
        import_module("edc_action_item.auths")
        import_module("edc_adverse_event.auths")
        import_module("edc_appointment.auths")
        import_module("edc_consent.auths")
        import_module("edc_pylabels.auths")
        import_module("edc_dashboard.auths")
        import_module("edc_data_manager.auths")
        import_module("edc_qareports.auths")
        import_module("edc_export.auths")
        import_module("edc_lab.auths")
        import_module("edc_lab_dashboard.auths")
        import_module("edc_review_dashboard.auths")
        import_module("edc_subject_dashboard.auths")
        import_module("edc_navbar.auths")
        import_module("edc_pharmacy.auths")
        import_module("edc_screening.auths")
        import_module("edc_qol.auths")
        import_module("edc_mnsi.auths")
        import_module("edc_unblinding.auths")
        import_module("edc_randomization.auths")
        import_module("meta_auth.auths")
        AuthUpdater(verbose=False)

    def setUp(self):
        self.user = User.objects.create_superuser("user_login", "u@example.com", "pass")

    def login(self, **kwargs):
        return login(self, redirect_url="home_url", **kwargs)

    @tag("webtest")
    def test_login(self):
        self.login(superuser=False, roles=[STAFF_ROLE])

    @tag("webtest")
    def test_ae(self):
        self.login(superuser=False, roles={STAFF_ROLE, AE_ROLE, TMG_ROLE})
        response = self.app.get(reverse("meta_ae:home_url"), user=self.user, status=302)
        response = response.follow()
        self.assertIn("META Adverse Events", response)
        self.app.get(reverse("edc_adverse_event:ae_home_url"), user=self.user, status=200)
        self.app.get(reverse("edc_adverse_event:tmg_home_url"), user=self.user, status=200)

    @skip("test_home_everyone")
    def test_home_everyone(self):
        self.login(superuser=False, roles=[STAFF_ROLE])
        response = self.app.get(reverse("home_url"), user=self.user, status=200)
        self.assertNotIn("Screening", response)
        self.assertNotIn("Subjects", response)
        self.assertNotIn("Specimens", response)
        self.assertNotIn("Adverse events", response)
        self.assertNotIn("Pharmacy", response)
        self.assertNotIn("Action items", response)
        self.assertNotIn("Export data", response)
        self.assertNotIn("Synchronization", response)

        self.assertIn("Switch sites", response)
        self.assertIn("Log out", response)

    @tag("webtest1")
    def test_home_auditor(self):
        self.login(superuser=False, roles=[STAFF_ROLE, AUDITOR_ROLE])
        response = self.app.get(reverse("home_url"), user=self.user, status=200)
        self.assertNotIn("Pharmacy", response)
        self.assertNotIn("Export data", response)
        self.assertNotIn("Synchronization", response)

        self.assertIn("Screening", response)
        self.assertIn("Subjects", response)
        self.assertIn("Specimens", response)
        self.assertIn("Adverse events", response)
        self.assertIn("TMG Reports", response)
        self.assertIn("Switch sites", response)
        self.assertIn("Log out", response)

        response = response.click(linkid="home_list_group_aetmg")
        self.assertIn("TMG Reports", response)

    @tag("webtest")
    def test_home_clinic(self):
        self.login(superuser=False, roles=[STAFF_ROLE, CLINICIAN_ROLE])
        response = self.app.get(reverse("home_url"), user=self.user, status=200)
        self.assertIn("Screening", response)
        self.assertIn("Subjects", response)
        self.assertIn("Specimens", response)
        self.assertIn("Adverse events", response)
        self.assertNotIn("TMG Reports", response)
        self.assertNotIn("Pharmacy", response)
        self.assertIn("Action items", response)
        self.assertNotIn("Export data", response)
        self.assertIn("Switch sites", response)
        self.assertIn("Log out", response)

    @skip("test_home_export")
    def test_home_export(self):
        self.login(superuser=False, roles=[STAFF_ROLE, DATA_EXPORTER_ROLE])
        response = self.app.get(reverse("home_url"), user=self.user, status=200)
        self.assertNotIn("Screening", response)
        self.assertNotIn("Subjects", response)
        self.assertNotIn("Specimens", response)
        self.assertNotIn("Adverse events", response)
        self.assertNotIn("TMG Reports", response)
        self.assertNotIn("Pharmacy", response)
        self.assertNotIn("Action items", response)
        self.assertIn("Export data", response)
        self.assertNotIn("Synchronization", response)
        self.assertIn("Switch sites", response)
        self.assertIn("Log out", response)

    @skip("test_home_tmg")
    def test_home_tmg(self):
        self.login(superuser=False, roles=[STAFF_ROLE, TMG_ROLE])
        response = self.app.get(reverse("home_url"), user=self.user, status=200)
        self.assertIn("Screening", response)
        self.assertIn("Subjects", response)
        self.assertNotIn("Specimens", response)
        self.assertIn("Adverse events", response)
        self.assertIn("TMG Reports", response)
        self.assertNotIn("Pharmacy", response)
        self.assertIn("Action items", response)
        self.assertNotIn("Export data", response)
        self.assertNotIn("Synchronization", response)
        self.assertIn("Switch sites", response)
        self.assertIn("Log out", response)
        response = response.click(linkid="home_list_group_aetmg")
        self.assertIn("TMG Reports", response)

    @skip("test_home_lab")
    def test_home_lab(self):
        self.login(superuser=False, roles=[STAFF_ROLE, LAB_TECHNICIAN_ROLE])
        response = self.app.get(reverse("home_url"), user=self.user, status=200)
        expected_labels = ["Specimens", "Switch sites", "Log out"]
        for label in self.menu_labels:
            if label in expected_labels:
                self.assertIn(label, response)
            else:
                self.assertNotIn(label, response)

    @skip
    @time_machine.travel(datetime(2019, 6, 11, 8, 00, tzinfo=ZoneInfo("UTC")))
    def test_screening_form_phase3(self):
        self.login(superuser=False, roles=[STAFF_ROLE, CLINICIAN_ROLE])
        site_randomizers._registry = {}
        site_randomizers.loaded = False
        site_randomizers.register(RandomizerPhaseThree)
        register_admin()

        part_one_data = deepcopy(get_part_one_eligible_options())
        part_one_data["report_datetime"] = get_utcnow()
        part_one_data.update(
            dict(
                report_datetime_0=part_one_data["report_datetime"].strftime("%Y-%m-%d"),
                report_datetime_1=part_one_data["report_datetime"].strftime("%H:%M"),
                continue_part_two=YES,
                site=Site.objects.get(id=settings.SITE_ID).id,
            )
        )
        (
            home_page,
            add_screening_part_two,
            screening_identifier,
        ) = self.webtest_for_screening_form_part_one(part_one_data)

        # travel 10 min into the future
        traveller = time_machine.travel(
            part_one_data["report_datetime"] + relativedelta(minutes=10)
        )
        traveller.start()
        part_two_data = deepcopy(
            get_part_two_eligible_options(
                report_datetime=get_utcnow() + relativedelta(minutes=1)
            )
        )
        part_two_data["appt_datetime"] = get_utcnow() + relativedelta(days=10)
        part_two_data.update(
            dict(
                part_two_report_datetime_0=part_two_data["part_two_report_datetime"].strftime(
                    "%Y-%m-%d"
                ),
                part_two_report_datetime_1=part_two_data["part_two_report_datetime"].strftime(
                    "%H:%M"
                ),
                appt_datetime_0=part_two_data["appt_datetime"].strftime("%Y-%m-%d"),
                appt_datetime_1=part_two_data["appt_datetime"].strftime("%H:%M"),
            )
        )
        (
            home_page,
            add_screening_part_three,
            screening_identifier,
        ) = self.webtest_for_screening_form_part_two(
            home_page, add_screening_part_two, screening_identifier, part_two_data
        )
        traveller.stop()

        # travel 10 days into the future
        traveller = time_machine.travel(part_two_data["appt_datetime"])
        traveller.start()
        part_three_data = deepcopy(
            get_part_three_eligible_options(report_datetime=get_utcnow())
        )

        traveller = time_machine.travel(part_two_data.get("part_three_report_datetime"))
        traveller.start()
        part_three_data.update(
            hba1c_datetime=part_three_data.get("part_three_report_datetime")
        )
        hba1c_datetime = part_three_data.get("hba1c_datetime")
        fbg_datetime = part_three_data.get("fbg_datetime")
        ogtt_datetime = part_three_data.get("ogtt_datetime")
        part_three_data.update(
            dict(
                part_three_report_datetime_0=part_three_data.get(
                    "part_three_report_datetime"
                ).strftime("%Y-%m-%d"),
                part_three_report_datetime_1=part_three_data.get(
                    "part_three_report_datetime"
                ).strftime("%H:%M"),
                fbg_datetime_0=fbg_datetime.strftime("%Y-%m-%d"),
                fbg_datetime_1=fbg_datetime.strftime("%H:%M"),
                ogtt_datetime_0=ogtt_datetime.strftime("%Y-%m-%d"),
                ogtt_datetime_1=ogtt_datetime.strftime("%H:%M"),
                hba1c_datetime_0=hba1c_datetime.strftime("%Y-%m-%d"),
                hba1c_datetime_1=hba1c_datetime.strftime("%H:%M"),
            )
        )
        (
            screening_listboard_page,
            screening_identifier,
        ) = self.webtest_for_screening_form_part_three(
            home_page, add_screening_part_three, screening_identifier, part_three_data
        )
        self.assertIn(screening_identifier, screening_listboard_page)
        self.assertIn("Consent", screening_listboard_page)

    def webtest_for_screening_form_part_one(self, part_one_data):
        home_page = self.app.get(reverse("home_url"), user=self.user, status=200)
        screening_listboard_page = home_page.click(description="Screening", index=1)
        add_screening_page = screening_listboard_page.click(
            description="Add Subject Screening"
        )
        # submit blank form
        response = add_screening_page.forms.get("screeningpartone_form").submit()
        self.assertIn("Please correct the errors below", response)
        # submit completed form
        for field, _ in add_screening_page.forms.get("screeningpartone_form").fields.items():
            try:
                add_screening_page.forms.get("screeningpartone_form")[field] = part_one_data[
                    field
                ]
            except KeyError as e:
                print(f"{field}. Got {e}.")
        page = add_screening_page.forms.get("screeningpartone_form").submit()
        soup = BeautifulSoup(page.content, "html.parser")
        errorlist = soup.find_all("ul", "errorlist")
        self.assertEqual([], errorlist)

        # redirects back to listboard
        url = reverse(screening_listboard_url)
        self.assertRedirects(page, url)

        # new screened subject is available
        obj = SubjectScreening.objects.all().last()
        screening_listboard_page = home_page.click(description="Screening", index=1)
        self.assertIn(obj.screening_identifier, screening_listboard_page)

        # shows P1 | P2 | P3 | PENDING
        self.assertIn("PENDING", screening_listboard_page)
        add_screening_part_two = screening_listboard_page.click(description="P2", index=0)
        self.assertEqual(add_screening_part_two.status_code, 200)
        return home_page, add_screening_part_two, obj.screening_identifier

    def webtest_for_screening_form_part_two(
        self, home_page, add_screening_part_two, screening_identifier, part_two_data
    ):
        # submit completed form
        for field, _ in add_screening_part_two.forms.get(
            "screeningparttwo_form"
        ).fields.items():
            try:
                add_screening_part_two.forms.get("screeningparttwo_form")[field] = (
                    part_two_data[field]
                )
            except KeyError:
                print(field)
        page = add_screening_part_two.forms.get("screeningparttwo_form").submit()
        soup = BeautifulSoup(page.content, "html.parser")
        errorlist = soup.find_all("ul", "errorlist")
        self.assertEqual([], errorlist)
        # redirects back to listboard
        obj = SubjectScreening.objects.get(screening_identifier=screening_identifier)
        url = reverse(screening_listboard_url, args=(obj.screening_identifier,))
        self.assertRedirects(page, url)
        # shows P1 | P2 | P3 | PENDING
        screening_listboard_page = home_page.click(description="Screening", index=1)
        self.assertIn("PENDING", screening_listboard_page)
        add_screening_part_three = screening_listboard_page.click(description="P3", index=0)
        self.assertEqual(add_screening_part_three.status_code, 200)
        return home_page, add_screening_part_three, screening_identifier

    def webtest_for_screening_form_part_three(
        self, home_page, add_screening_part_three, screening_identifier, part_three_data
    ):
        # submit completed form
        for field, _ in add_screening_part_three.forms.get(
            "screeningpartthree_form"
        ).fields.items():
            try:
                add_screening_part_three.forms.get("screeningpartthree_form")[field] = (
                    part_three_data[field]
                )
            except KeyError:
                print(field)
        page = add_screening_part_three.forms.get("screeningpartthree_form").submit()
        soup = BeautifulSoup(page.content, "html.parser")
        errorlist = soup.find_all("ul", "errorlist")
        self.assertEqual([], errorlist)
        # redirects back to listboard
        obj = SubjectScreening.objects.all().last()
        url = reverse(screening_listboard_url, args=(obj.screening_identifier,))
        self.assertRedirects(page, url)
        # shows P1 | P2 | P3 | Consent
        screening_listboard_page = home_page.click(description="Screening", index=1)
        self.assertIn("Consent", screening_listboard_page)
        return screening_listboard_page, screening_identifier

    def get_subject_screening(self, **kwargs):
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        part_two_eligible_options = deepcopy(get_part_two_eligible_options())
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        obj = ScreeningPartOne(**part_one_eligible_options)
        obj.save()
        self.screening_identifier = obj.screening_identifier
        obj = ScreeningPartTwo.objects.get(screening_identifier=self.screening_identifier)
        for k, v in part_two_eligible_options.items():
            setattr(obj, k, v)
        obj.save()
        obj = ScreeningPartThree.objects.get(screening_identifier=self.screening_identifier)
        for k, v in part_three_eligible_options.items():
            setattr(obj, k, v)
        obj.save()
        return obj

    @tag("webtest3")
    @skip("-")
    def test_to_subject_dashboard(self):
        add_or_update_django_sites(apps=django_apps, single_sites=all_sites)
        self.login(superuser=False, roles=[STAFF_ROLE, CLINICIAN_ROLE], sites=[10, 20, 30])

        subject_screening = self.get_subject_screening()

        home_page = self.app.get(reverse("home_url"), user=self.user, status=200)
        screening_listboard_page = home_page.click(description="Screening", index=1)

        add_subjectconsent_page = screening_listboard_page.click(
            description="Consent", index=1
        )
        # submit blank form
        response = add_subjectconsent_page.form.submit()
        self.assertIn("Please correct the errors below", response)

        consents = site_consents.get_consents_by_model("meta_consent.subjectconsentv1")
        consent_datetime = consents[0].start + relativedelta(months=1)
        subject_consent = baker.make_recipe(
            "meta_consent.subjectconsentv1",
            screening_identifier=subject_screening.screening_identifier,
            dob=(
                consent_datetime - relativedelta(years=subject_screening.age_in_years)
            ).date(),
            first_name="Melissa",
            last_name="Rodriguez",
            initials="MR",
            consent_datetime=consent_datetime,
        )

        home_page = self.app.get(reverse("home_url"), user=self.user, status=200)
        screening_listboard_page = home_page.click(description="Screening", index=1)

        self.assertIn("Dashboard", screening_listboard_page)
        self.assertIn(
            f"subjectscreening_change_{subject_screening.screening_identifier}",
            screening_listboard_page,
        )

        home_page = self.app.get(reverse("home_url"), user=self.user, status=200)
        subject_listboard_page = home_page.click(description="Subjects", index=1)

        self.assertIn(subject_consent.subject_identifier, subject_listboard_page)

        href = reverse(
            "meta_dashboard:subject_dashboard_url",
            kwargs={"subject_identifier": subject_consent.subject_identifier},
        )
        subject_dashboard_page = subject_listboard_page.click(href=href)

        self.assertEqual(subject_dashboard_page.status_code, 200)

        # on subject_dashboard
        # assert all appointment are showing
        subject_identifier = subject_consent.subject_identifier
        appointments = Appointment.objects.filter(
            subject_identifier=subject_identifier
        ).order_by("appt_datetime")
        for appointment in appointments:
            self.assertIn(appointment.visit_code, subject_dashboard_page)

        # start appointment 1000
        page = subject_dashboard_page.click(linkid="start_btn_1000_0")
        page.form["appt_status"] = IN_PROGRESS_APPT
        page.form["appt_reason"] = SCHEDULED_APPT
        subject_dashboard_page = page.form.submit()
        self.assertEqual(subject_dashboard_page.status_code, 302)
        self.assertEqual(
            subject_dashboard_page.url,
            f"/subject/subject_dashboard/{subject_identifier}/",
        )

        subject_dashboard_page = self.app.get(
            subject_dashboard_page.url, user=self.user, status=200
        )

        # start visit 1000
        self.assertIn(" Start ", subject_dashboard_page)
        subject_visit_page = subject_dashboard_page.click(
            linkid=(
                f"start_btn_{appointments[0].visit_code}_"
                f"{appointments[0].visit_code_sequence}"
            )
        )
        subject_visit_page.form["info_source"] = "patient"
        subject_dashboard_page = subject_visit_page.form.submit()

        url = f"/subject/subject_dashboard/{subject_identifier}/" f"{str(appointments[0].pk)}/"
        self.assertEqual(subject_dashboard_page.status_code, 302)
        self.assertEqual(subject_dashboard_page.url, url)

        subject_dashboard_page = self.app.get(
            reverse(
                "meta_dashboard:subject_dashboard_url",
                kwargs=dict(
                    subject_identifier=subject_identifier,
                    appointment=str(appointments[0].id),
                ),
            ),
            user=self.user,
            status=200,
        )

        self.assertIn("CRFs", subject_dashboard_page)
        self.assertIn("Requisitions", subject_dashboard_page)

    @tag("webtest")
    def test_follow_urls(self):
        """Follows any url that can be reversed without kwargs."""
        self.login(superuser=False, roles=[CLINICIAN_ROLE])
        for url_name in url_names.registry.values():
            sys.stdout.write(style.MIGRATE_HEADING(f" - '{url_name}' ...\r"))
            try:
                url = reverse(url_name)
            except NoReverseMatch:
                sys.stdout.write(
                    style.ERROR(
                        f" - '{url_name}'. Got `NoReverseMatch` "
                        f"when reversed without kwargs.\n"
                    )
                )
            else:
                try:
                    self.app.get(url, user=self.user, status=200)
                except AppError as e:
                    sys.stdout.write(style.ERROR(f" - '{url_name}'. Got `AppError`: {e}\n"))
                else:
                    sys.stdout.write(style.SUCCESS(f" - '{url_name}'->{url}\n"))



# Generated by Django 5.0.8 on 2024-08-28 23:50

import django_db_views.migration_functions
import django_db_views.operations
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0045_auto_20240829_0248"),
    ]

    operations = [
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'onstudy_missing_values_view' as `report_model` from (SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No HIV Diagnosis date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.hiv_diagnosis_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL value or VL date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load IS NULL OR crf.viral_load_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No CD4 value or CD4 date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.cd4 IS NULL OR crf.cd4_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date but previous ARV is YES' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.has_previous_arv_regimen = 'Yes' AND crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'Other current ARV regimen missing' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id LEFT JOIN meta_lists_arvregimens AS arvregimen ON crf.current_arv_regimen_id = arvregimen.id WHERE crf.other_current_arv_regimen IS NULL AND arvregimen.name = 'OTHER' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.glucose' AS label_lower, 'FBG/OGTT: missing OGTT' AS label, COUNT(*) AS records FROM meta_subject_glucose AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE (NOT crf.fbg_value IS NULL AND crf.ogtt_value IS NULL) AND ogtt_performed = 'Yes' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified) as A ORDER BY subject_identifier, site_id",
                "onstudy_missing_values_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'onstudy_missing_values_view' as `report_model` from (SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No HIV Diagnosis date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.hiv_diagnosis_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL value' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No CD4 value' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.cd4 IS NULL OR crf.cd4_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date but previous ARV is YES' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.has_previous_arv_regimen = 'Yes' AND crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'Other current ARV regimen missing' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id LEFT JOIN meta_lists_arvregimens AS arvregimen ON crf.current_arv_regimen_id = arvregimen.id WHERE crf.other_current_arv_regimen IS NULL AND arvregimen.name = 'OTHER' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.glucose' AS label_lower, 'FBG/OGTT: missing OGTT' AS label, COUNT(*) AS records FROM meta_subject_glucose AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE (NOT crf.fbg_value IS NULL AND crf.ogtt_value IS NULL) AND ogtt_performed = 'Yes' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified) as A ORDER BY subject_identifier, site_id",
                "onstudy_missing_values_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, 'onstudy_missing_values_view' as report_model from (SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No HIV Diagnosis date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.hiv_diagnosis_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL value or VL date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load IS NULL OR crf.viral_load_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No CD4 value or CD4 date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.cd4 IS NULL OR crf.cd4_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date but previous ARV is YES' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.has_previous_arv_regimen = 'Yes' AND crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'Other current ARV regimen missing' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id LEFT JOIN meta_lists_arvregimens AS arvregimen ON crf.current_arv_regimen_id = arvregimen.id WHERE crf.other_current_arv_regimen IS NULL AND arvregimen.name = 'OTHER' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.glucose' AS label_lower, 'FBG/OGTT: missing OGTT' AS label, COUNT(*) AS records FROM meta_subject_glucose AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE (NOT crf.fbg_value IS NULL AND crf.ogtt_value IS NULL) AND ogtt_performed = 'Yes' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified) as A ORDER BY subject_identifier, site_id",
                "onstudy_missing_values_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, 'onstudy_missing_values_view' as report_model from (SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No HIV Diagnosis date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.hiv_diagnosis_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL value' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No CD4 value' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.cd4 IS NULL OR crf.cd4_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date but previous ARV is YES' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.has_previous_arv_regimen = 'Yes' AND crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'Other current ARV regimen missing' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id LEFT JOIN meta_lists_arvregimens AS arvregimen ON crf.current_arv_regimen_id = arvregimen.id WHERE crf.other_current_arv_regimen IS NULL AND arvregimen.name = 'OTHER' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.glucose' AS label_lower, 'FBG/OGTT: missing OGTT' AS label, COUNT(*) AS records FROM meta_subject_glucose AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE (NOT crf.fbg_value IS NULL AND crf.ogtt_value IS NULL) AND ogtt_performed = 'Yes' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified) as A ORDER BY subject_identifier, site_id",
                "onstudy_missing_values_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, datetime() as created, 'onstudy_missing_values_view' as report_model from (SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No HIV Diagnosis date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.hiv_diagnosis_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL value or VL date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load IS NULL OR crf.viral_load_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No CD4 value or CD4 date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.cd4 IS NULL OR crf.cd4_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date but previous ARV is YES' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.has_previous_arv_regimen = 'Yes' AND crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'Other current ARV regimen missing' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id LEFT JOIN meta_lists_arvregimens AS arvregimen ON crf.current_arv_regimen_id = arvregimen.id WHERE crf.other_current_arv_regimen IS NULL AND arvregimen.name = 'OTHER' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.glucose' AS label_lower, 'FBG/OGTT: missing OGTT' AS label, COUNT(*) AS records FROM meta_subject_glucose AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE (NOT crf.fbg_value IS NULL AND crf.ogtt_value IS NULL) AND ogtt_performed = 'Yes' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified) as A ORDER BY subject_identifier, site_id",
                "onstudy_missing_values_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, uuid() as id, datetime() as created, 'onstudy_missing_values_view' as report_model from (SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No HIV Diagnosis date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.hiv_diagnosis_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL value' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No CD4 value' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.cd4 IS NULL OR crf.cd4_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date but previous ARV is YES' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.has_previous_arv_regimen = 'Yes' AND crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'Other current ARV regimen missing' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id LEFT JOIN meta_lists_arvregimens AS arvregimen ON crf.current_arv_regimen_id = arvregimen.id WHERE crf.other_current_arv_regimen IS NULL AND arvregimen.name = 'OTHER' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.glucose' AS label_lower, 'FBG/OGTT: missing OGTT' AS label, COUNT(*) AS records FROM meta_subject_glucose AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE (NOT crf.fbg_value IS NULL AND crf.ogtt_value IS NULL) AND ogtt_performed = 'Yes' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified) as A ORDER BY subject_identifier, site_id",
                "onstudy_missing_values_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
    ]



# Generated by Django 3.2.13 on 2022-09-14 02:47

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_consent", "0014_alter_subjectconsent_managers"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalsubjectconsent",
            name="ethnicity",
            field=models.CharField(
                editable=False, help_text="fromm screening", max_length=15, null=True
            ),
        ),
        migrations.AddField(
            model_name="subjectconsent",
            name="ethnicity",
            field=models.CharField(
                editable=False, help_text="fromm screening", max_length=15, null=True
            ),
        ),
    ]



import os

from celery import Celery

# load settings
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "meta_edc.settings.debug")

# config celery
app = Celery("meta_edc")
app.config_from_object("django.conf:settings", namespace="CELERY")
app.autodiscover_tasks()

app.conf.task_routes = {
    "*": {"queue": "uat_queue"},
}


@app.task(bind=True)
def debug_task(self):
    print("Request: {0!r}".format(self.request))



# Generated by Django 5.0.8 on 2024-08-21 13:45

import uuid

import _socket
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_crf.model_mixins.crf_status_model_mixin
import edc_model.validators.date
import edc_model_fields.fields.other_charfield
import edc_protocol.validators
import edc_utils.date
import edc_visit_tracking.managers
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0208_birthoutcomes_crf_status_and_more"),
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RenameModel(
            old_name="historicaldmdiagnosis",
            new_name="historicaldmendpoint",
        ),
        migrations.RenameModel(old_name="dmdiagnosis", new_name="dmendpoint"),
    ]



from django.test import TestCase
from edc_sites.exceptions import InvalidSiteError
from edc_sites.site import sites


class SiteTests(TestCase):
    def test_all(self):
        self.assertEqual(sites.get_by_attr("name", "hindu_mandal").site_id, 10)
        self.assertEqual(sites.get_by_attr("name", "amana").site_id, 20)

    def test_bad(self):
        self.assertRaises(InvalidSiteError, sites.get_by_attr, "name", "erik")



# Generated by Django 4.2.11 on 2024-05-16 00:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "meta_subject",
            "0200_rename_fasting_duration_minutes_bloodresultsins_fasting_duration_delta_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsins",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="glucose",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="glucosefbg",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsins",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucosefbg",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
    ]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_subject_admin
from ..forms import PatientHistoryForm
from ..models import PatientHistory
from .fieldsets import get_hiv_fieldset, get_htn_fieldset, get_other_history_fieldset
from .modeladmin import CrfModelAdminMixin


@admin.register(PatientHistory, site=meta_subject_admin)
class PatientHistoryAdmin(CrfModelAdminMixin, SimpleHistoryAdmin):
    form = PatientHistoryForm

    autocomplete_fields = ["current_arv_regimen", "previous_arv_regimen"]
    fieldsets = [
        (None, {"fields": ("subject_visit", "report_datetime")}),
        ("Part 1: Symptoms", {"fields": ("symptoms", "other_symptoms")}),
        get_hiv_fieldset(part="2"),
        get_htn_fieldset(part="3"),
        get_other_history_fieldset(part="4"),
        crf_status_fieldset,
        audit_fieldset_tuple,
    ]

    radio_fields = {
        "current_arv_regimen": admin.VERTICAL,
        "current_smoker": admin.VERTICAL,
        "dm_in_family": admin.VERTICAL,
        "dyslipidaemia_diagnosis": admin.VERTICAL,
        "dyslipidaemia_rx": admin.VERTICAL,
        "former_smoker": admin.VERTICAL,
        "has_previous_arv_regimen": admin.VERTICAL,
        "htn_diagnosis": admin.VERTICAL,
        "on_dyslipidaemia_treatment": admin.VERTICAL,
        "on_htn_treatment": admin.VERTICAL,
        "on_oi_prophylaxis": admin.VERTICAL,
        "previous_arv_regimen": admin.VERTICAL,
        "taking_statins": admin.VERTICAL,
    }

    filter_horizontal = (
        "symptoms",
        "oi_prophylaxis",
        "dm_symptoms",
        "htn_treatment",
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from bs4 import BeautifulSoup
from django import template
from edc_constants.constants import TBD
from edc_dashboard.url_names import url_names
from edc_dashboard.utils import get_bootstrap_version

from meta_consent.models import SubjectConsent
from meta_dashboard.view_utils import (
    SubjectScreeningPartOneButton,
    SubjectScreeningPartThreeButton,
    SubjectScreeningPartTwoButton,
)
from meta_screening.eligibility import get_display_label
from meta_screening.models import (
    ScreeningPartOne,
    ScreeningPartThree,
    ScreeningPartTwo,
    SubjectScreening,
)

if TYPE_CHECKING:
    pass


register = template.Library()


@register.inclusion_tag(
    f"meta_dashboard/bootstrap{get_bootstrap_version()}/" f"buttons/eligibility_button.html"
)
def eligibility_button(subject_screening: SubjectScreening):
    comment = []
    tooltip = None
    if subject_screening.reasons_ineligible:
        comment = subject_screening.reasons_ineligible.split("|")
        comment = list(set(comment))
        comment.sort()
    display_label = get_display_label(subject_screening)
    soup = BeautifulSoup(display_label, features="html.parser")
    return dict(
        eligible=subject_screening.eligible,
        eligible_final=subject_screening.eligible,
        display_label=soup.get_text(),
        comment=comment,
        tooltip=tooltip,
        TBD=TBD,
    )


@register.inclusion_tag(
    f"meta_dashboard/bootstrap{get_bootstrap_version()}/buttons/add_consent_button.html",
    takes_context=True,
)
def render_consent_button(context, subject_screening: SubjectScreening):
    title = ["Consent subject to participate."]
    cdef = subject_screening.consent_definition
    return dict(
        perms=context["perms"],
        screening_identifier=subject_screening.screening_identifier,
        # href=model_wrapper.consent.href,
        consent_version=cdef.version,
        title=" ".join(title),
    )


def refusal_button(context, subject_refusal):
    title = ["Capture subject's primary reason for not joining."]

    return dict(
        perms=context["perms"],
        # href=subject_refusal_model_wrapper.href,
        title=" ".join(title),
    )


@register.inclusion_tag(
    f"edc_listboard/bootstrap{get_bootstrap_version()}/buttons/dashboard_button.html"
)
def render_dashboard_button(subject_consent: SubjectConsent):
    subject_dashboard_url = url_names.get("subject_dashboard_url")
    return dict(
        subject_dashboard_url=subject_dashboard_url,
        subject_identifier=subject_consent.subject_identifier,
    )


@register.inclusion_tag(
    f"edc_subject_dashboard/bootstrap{get_bootstrap_version()}/buttons/forms_button.html",
    takes_context=True,
)
def render_screening_part_one_button(context, subject_screening: ScreeningPartOne) -> dict:
    btn = SubjectScreeningPartOneButton(
        user=context["request"].user,
        model_obj=subject_screening,
        current_site=context["request"].site,
    )
    return dict(btn=btn)


@register.inclusion_tag(
    f"edc_subject_dashboard/bootstrap{get_bootstrap_version()}/buttons/forms_button.html",
    takes_context=True,
)
def render_screening_part_two_button(context, subject_screening: ScreeningPartTwo) -> dict:
    btn = SubjectScreeningPartTwoButton(
        user=context["request"].user,
        model_obj=subject_screening,
        current_site=context["request"].site,
    )
    return dict(btn=btn)


@register.inclusion_tag(
    f"edc_subject_dashboard/bootstrap{get_bootstrap_version()}/buttons/forms_button.html",
    takes_context=True,
)
def render_screening_part_three_button(context, subject_screening: ScreeningPartThree) -> dict:
    btn = SubjectScreeningPartThreeButton(
        user=context["request"].user,
        model_obj=subject_screening,
        current_site=context["request"].site,
    )
    return dict(btn=btn)



# Generated by Django 5.0.8 on 2024-08-21 23:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0210_remove_dmdxresult_dm_diagnosis_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="dmendpoint",
            name="endpoint_reached",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If YES, the EDC will check for the patient on the Endpoints report.",
                max_length=15,
                null=True,
                verbose_name="Was the patient referred because the diabetes endpoint was reached?",
            ),
        ),
        migrations.AddField(
            model_name="historicaldmendpoint",
            name="endpoint_reached",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If YES, the EDC will check for the patient on the Endpoints report.",
                max_length=15,
                null=True,
                verbose_name="Was the patient referred because the diabetes endpoint was reached?",
            ),
        ),
        migrations.AlterField(
            model_name="dmendpoint",
            name="dx_date",
            field=models.DateField(null=True, verbose_name="Date endpoint reached"),
        ),
        migrations.AlterField(
            model_name="historicaldmendpoint",
            name="dx_date",
            field=models.DateField(null=True, verbose_name="Date endpoint reached"),
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-30 19:57
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations
from edc_constants.constants import NOT_APPLICABLE
from tqdm import tqdm


def update_investigator_ae_classification(apps, schema_editor):
    ae_tmg_cls = apps.get_model("meta_ae", "aetmg")
    ae_classification_cls = apps.get_model("edc_adverse_event", "aeclassification")
    try:
        ae_classification = ae_classification_cls.objects.get(name=NOT_APPLICABLE)
    except ObjectDoesNotExist:
        pass
    else:
        total = ae_tmg_cls.objects.filter(investigator_ae_classification__isnull=True).count()
        for obj in tqdm(
            ae_tmg_cls.objects.filter(investigator_ae_classification__isnull=True), total=total
        ):
            obj.investigator_ae_classification = ae_classification
            obj.save_base(update_fields=["investigator_ae_classification"])


class Migration(migrations.Migration):
    dependencies = [
        ("meta_ae", "0016_rename_narrative_aetmg_investigator_narrative_and_more"),
    ]

    operations = [migrations.RunPython(update_investigator_ae_classification)]



# Generated by Django 5.0.4 on 2024-04-18 23:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0058_dmreferral_referral_note_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="reason",
            field=models.CharField(
                choices=[
                    ("pregnancy", "Pregnancy"),
                    ("sae", "Participant is experiencing a serious adverse event"),
                    (
                        "clinician",
                        "Other condition that justifies discontinuation of treatment in the clinician's opinion (specify below)",
                    ),
                    ("investigator", " Investigator decision"),
                    ("referral", "Referral to Diabetes clinic"),
                    ("patient", "Patient decision"),
                    ("OTHER", "Other reason (specify below)"),
                ],
                max_length=25,
                verbose_name="Reason for stopping study medication",
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="reason",
            field=models.CharField(
                choices=[
                    ("pregnancy", "Pregnancy"),
                    ("sae", "Participant is experiencing a serious adverse event"),
                    (
                        "clinician",
                        "Other condition that justifies discontinuation of treatment in the clinician's opinion (specify below)",
                    ),
                    ("investigator", " Investigator decision"),
                    ("referral", "Referral to Diabetes clinic"),
                    ("patient", "Patient decision"),
                    ("OTHER", "Other reason (specify below)"),
                ],
                max_length=25,
                verbose_name="Reason for stopping study medication",
            ),
        ),
    ]



from typing import Tuple

from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import ActionItemModelAdminMixin, action_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import meta_prn_admin
from ..forms import OffStudyMedicationForm
from ..models import OffStudyMedication


@admin.register(OffStudyMedication, site=meta_prn_admin)
class OffStudyMedicationAdmin(
    SiteModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    ActionItemModelAdminMixin,
    SimpleHistoryAdmin,
):
    form = OffStudyMedicationForm

    fieldsets = (
        (None, {"fields": ("subject_identifier", "report_datetime")}),
        (
            "Loss to followup",
            {
                "fields": (
                    "medications",
                    "stop_date",
                    "last_dose_date",
                    "reason",
                    "reason_other",
                    "comment",
                )
            },
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "reason": admin.VERTICAL,
    }

    filter_horizontal = ("medications",)

    search_fields = ("subject_identifier",)

    def get_list_display(self, request) -> Tuple[str, ...]:
        list_display = super().get_list_display(request)
        custom_fields = (
            "subject_identifier",
            "dashboard",
            "decision_date",
            "last_known_dose_date",
        )
        return custom_fields + tuple(f for f in list_display if f not in custom_fields)

    def get_list_filter(self, request) -> Tuple[str, ...]:
        list_filter = super().get_list_filter(request)
        custom_fields = (
            "medications",
            "reason",
            "stop_date",
            "last_dose_date",
        )
        return custom_fields + tuple(f for f in list_filter if f not in custom_fields)

    @admin.display(description="decision date", ordering="stop_date")
    def decision_date(self, obj=None):
        return obj.stop_date

    @admin.display(description="last dose date", ordering="last_dose_date")
    def last_known_dose_date(self, obj=None):
        return obj.last_dose_date



# Generated by Django 5.0.8 on 2024-08-28 19:54

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("edc_action_item", "0037_remove_actionitem_reference_model_and_more"),
        ("meta_subject", "0212_auto_20240827_2222"),
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RenameModel(
            old_name="BloodResultsLipid",
            new_name="BloodResultsLipids",
        ),
        migrations.RenameModel(
            old_name="HistoricalBloodResultsLipid",
            new_name="HistoricalBloodResultsLipids",
        ),
        migrations.RenameIndex(
            model_name="bloodresultslipids",
            new_name="meta_subjec_subject_eff3e6_idx",
            old_name="meta_subjec_subject_296de9_idx",
        ),
        migrations.RenameIndex(
            model_name="bloodresultslipids",
            new_name="meta_subjec_subject_49b971_idx",
            old_name="meta_subjec_subject_9692a2_idx",
        ),
    ]



"""
A settings file with the bare minimum attributes.
"""

import os
from pathlib import Path

import environ
from multisite import SiteID

APP_NAME = "meta_edc"
BASE_DIR = str(Path(os.path.dirname(os.path.abspath(__file__))).parent.parent)
ENV_DIR = str(Path(os.path.dirname(os.path.abspath(__file__))).parent.parent)

env = environ.Env()
env.read_env(os.path.join(ENV_DIR, ".env"))

EDC_SITES_MODULE_NAME = env.str("EDC_SITES_MODULE_NAME")
ETC_DIR = env.str("DJANGO_ETC_FOLDER")
SECRET_KEY = "blahblahblah"  # nosec B105
ALLOWED_HOSTS = ["*"]
SITE_ID = SiteID(default=1)  # 1 is not a site



# Generated by Django 5.0.8 on 2024-08-23 14:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0036_historicalmissingogttnote_fasting_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalmissingogttnote",
            name="result_status",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=25,
                verbose_name="Is the OGTT result available",
            ),
        ),
        migrations.AddField(
            model_name="missingogttnote",
            name="result_status",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=25,
                verbose_name="Is the OGTT result available",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmissingogttnote",
            name="status",
            field=models.CharField(
                choices=[("COMPLETE", "Complete"), ("not_available", "Not available")],
                default="done",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="missingogttnote",
            name="status",
            field=models.CharField(
                choices=[("COMPLETE", "Complete"), ("not_available", "Not available")],
                default="done",
                max_length=25,
            ),
        ),
    ]



# Generated by Django 3.0.6 on 2020-05-28 12:17
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations
from django.db.migrations import RunPython
from edc_ltfu.constants import LOST_TO_FOLLOWUP


def update_ltfu(apps, schema_editor):
    offstudy_reasons = apps.get_model("meta_lists.offstudyreasons")
    try:
        obj = offstudy_reasons.objects.get(name="lost_to_followup")
    except ObjectDoesNotExist:
        pass
    else:
        obj.name = LOST_TO_FOLLOWUP
        obj.save_base(update_fields=["name"])


class Migration(migrations.Migration):
    dependencies = [
        ("meta_lists", "0007_auto_20200516_2356"),
    ]

    operations = [RunPython(update_ltfu)]



# Generated by Django 3.2.11 on 2022-03-09 19:38

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0099_auto_20220309_2218"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalurinepregnancy",
            name="bhcg_value",
            field=models.CharField(
                choices=[
                    ("POS", "Positive"),
                    ("NEG", "Negative"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="Urine βhCG result",
            ),
        ),
        migrations.AlterField(
            model_name="urinepregnancy",
            name="bhcg_value",
            field=models.CharField(
                choices=[
                    ("POS", "Positive"),
                    ("NEG", "Negative"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="Urine βhCG result",
            ),
        ),
    ]



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_vitals.models import WeightField


class VitalsFieldsModelMixin(models.Model):
    weight = WeightField(null=True)

    # 10
    heart_rate = models.IntegerField(
        verbose_name="Heart rate:",
        validators=[MinValueValidator(30), MaxValueValidator(200)],
        help_text="BPM",
    )

    # 11
    respiratory_rate = models.IntegerField(
        verbose_name="Respiratory rate:",
        validators=[MinValueValidator(6), MaxValueValidator(50)],
        help_text="breaths/min",
        null=True,
        blank=True,
    )

    # 12
    oxygen_saturation = models.IntegerField(
        verbose_name="Oxygen saturation:",
        validators=[MinValueValidator(1), MaxValueValidator(999)],
        help_text="%",
        null=True,
        blank=True,
    )

    temperature = models.DecimalField(
        verbose_name="Temperature:",
        validators=[MinValueValidator(30), MaxValueValidator(45)],
        decimal_places=1,
        max_digits=3,
        help_text="in degrees Celcius",
    )

    class Meta:
        abstract = True



from django import forms
from edc_action_item.forms import ActionItemCrfFormMixin
from edc_crf.modelform_mixins import CrfModelFormMixin

from ..form_validators import FollowupExaminationFormValidator
from ..models import FollowupExamination


class FollowupExaminationForm(CrfModelFormMixin, ActionItemCrfFormMixin, forms.ModelForm):
    form_validator_cls = FollowupExaminationFormValidator

    class Meta(ActionItemCrfFormMixin.Meta):
        model = FollowupExamination
        fields = "__all__"



from edc_pharmacy.models import Stock
from edc_pharmacy.utils import format_qty
from edc_sites.site import sites as site_sites
from reportlab.graphics.shapes import Drawing, Group, String
from reportlab.pdfbase.pdfmetrics import stringWidth


def draw_label_for_subject_with_barcode(
    barcode_cls,
    barcode_opts: dict,
    label: Drawing,
    width: int | float,
    height: int | float,
    obj: Stock,
) -> Drawing:
    """Callable to draw a single study medication label given a model
    instance `obj`
    """
    try:
        single_site = site_sites.get(
            obj.allocation.stock_request_item.stock_request.location.site.id
        )
    except AttributeError:
        label.add(String(15, height - 20, "Error: Unable to generate label", fontSize=10))
        label.add(String(15, height - 40, str(obj.code), fontSize=10))
        label.add(String(15, height - 60, str(obj.location), fontSize=10))
    else:
        br = barcode_cls(**barcode_opts)
        br.value = obj.code
        br.x = width - 110
        br.y = 40
        label.add(br)

        label.add(String(15, height - 35, single_site.title, fontSize=10))

        label.add(String(15, height - 20, "META III Study", fontSize=12))
        label.add(String(15, height - 55, "Dawa kwa ajili ya utafiti", fontSize=10))
        label.add(String(15, height - 67, "wa META III.", fontSize=10))
        label.add(String(15, height - 79, "Meza vidonge vinne usiku tu.", fontSize=10))

        text_group = Group()
        subject_identifier = (
            obj.allocation.registered_subject.subject_identifier or "999-99-99999-9"
        )
        text_width = stringWidth(subject_identifier, "Helvetica", 12)
        text_string = String(
            width - text_width - 10, height - 20, subject_identifier, fontSize=12
        )
        text_group.add(text_string)

        qty_text = f"{format_qty(obj.container.qty, obj.container)} tabs"
        text_width = stringWidth(qty_text, "Helvetica", 10)
        text_string = String(width - text_width - 10, height - 95, qty_text, fontSize=10)
        text_group.add(text_string)

        label.add(text_group)

    return label



from edc_model.models import BaseUuidModel
from edc_transfer.model_mixins import SubjectTransferModelMixin


class SubjectTransfer(
    SubjectTransferModelMixin,
    BaseUuidModel,
):
    class Meta(SubjectTransferModelMixin.Meta, BaseUuidModel.Meta):
        pass



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin

from ...form_validators import HealthEconomicsFormValidator
from ...models import HealthEconomicsSimple


class HealthEconomicsSimpleForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = HealthEconomicsFormValidator

    class Meta:
        model = HealthEconomicsSimple
        fields = "__all__"



from __future__ import annotations

from typing import Type

from django.contrib import admin
from django.contrib.admin import SimpleListFilter
from django_audit_fields import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_subject_admin
from ..forms import GlucoseFbgForm
from ..models import GlucoseFbg
from .list_filters import GlucoseListFilter
from .modeladmin import CrfModelAdminMixin


@admin.register(GlucoseFbg, site=meta_subject_admin)
class GlucoseFbgAdmin(CrfModelAdminMixin, SimpleHistoryAdmin):
    form = GlucoseFbgForm

    fieldsets = [
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Fasting",
            {
                "fields": (
                    "fasting",
                    "fasting_duration_str",
                )
            },
        ),
        (
            "Blood Glucose",
            {
                "fields": (
                    "fbg_performed",
                    "fbg_not_performed_reason",
                    "fbg_datetime",
                    "fbg_value",
                    "fbg_units",
                )
            },
        ),
        (
            "Confirmation appointment",
            {
                "description": (
                    "If blood glucose value is >= 7.0 mmol/L, schedule an "
                    "appointment within 1 week to confirm"
                ),
                "fields": ("repeat_fbg_date",),
            },
        ),
        crf_status_fieldset,
        audit_fieldset_tuple,
    ]

    radio_fields = {
        "fasting": admin.VERTICAL,
        "fbg_units": admin.VERTICAL,
        "fbg_performed": admin.VERTICAL,
    }

    @admin.display(description="FBG", ordering="fbg_value")
    def fbg(self, obj=None):
        return obj.fbg_value

    def get_list_display(self, request) -> tuple[str, ...]:
        list_display = super().get_list_display(request)
        list_display = list(list_display)
        # list_display.insert(3, "ogtt")
        list_display.insert(3, "fbg")
        list_display = tuple(list_display)
        return list_display

    def get_list_filter(self, request) -> tuple[str | Type[SimpleListFilter], ...]:
        list_filter = super().get_list_filter(request)
        list_filter = list(list_filter)
        list_filter.insert(2, GlucoseListFilter)
        list_filter = tuple(list_filter)
        return list_filter



from django import forms
from edc_adverse_event.modelform_mixins import DeathReportTmgModelFormMixin

from ..models import DeathReportTmgSecond


class DeathReportTmgSecondForm(DeathReportTmgModelFormMixin, forms.ModelForm):
    class Meta(DeathReportTmgModelFormMixin.Meta):
        model = DeathReportTmgSecond
        fields = "__all__"



# Generated by Django 4.2.11 on 2024-05-16 01:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0203_alter_bloodresultsins_fasting_duration_delta_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="glucosefbg",
            name="repeat_fbg_date",
            field=models.DateField(
                blank=True, help_text="Date should be within 1 week of report date", null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalglucosefbg",
            name="repeat_fbg_date",
            field=models.DateField(
                blank=True, help_text="Date should be within 1 week of report date", null=True
            ),
        ),
    ]



# Generated by Django 3.2.6 on 2021-09-09 23:34


from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0083_auto_20210823_1620"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalmnsi",
            name="ankle_reflexes_left_foot",
            field=models.CharField(
                choices=[
                    ("present", "Present"),
                    ("present_with_reinforcement", "Present/Reinforcement"),
                    ("absent", "Absent"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Ankle reflexes, LEFT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="ankle_reflexes_right_foot",
            field=models.CharField(
                choices=[
                    ("present", "Present"),
                    ("present_with_reinforcement", "Present/Reinforcement"),
                    ("absent", "Absent"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Ankle reflexes, RIGHT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="examined_left_foot",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If the MNSI assessment was not performed, respond with `not applicable`.",
                max_length=15,
                verbose_name="Was the LEFT foot examined?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="examined_right_foot",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If the MNSI assessment was not performed, respond with `not applicable`.",
                max_length=15,
                verbose_name="Was the RIGHT foot examined?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="monofilament_left_foot",
            field=models.CharField(
                choices=[
                    ("NORMAL", "Normal"),
                    ("reduced", "Reduced"),
                    ("absent", "Absent"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Monofilament, LEFT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="monofilament_right_foot",
            field=models.CharField(
                choices=[
                    ("NORMAL", "Normal"),
                    ("reduced", "Reduced"),
                    ("absent", "Absent"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Monofilament, RIGHT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="normal_appearance_left_foot",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=15,
                verbose_name="Does LEFT foot appear normal?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="normal_appearance_right_foot",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=15,
                verbose_name="Does RIGHT foot appear normal?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="ulceration_left_foot",
            field=models.CharField(
                choices=[
                    ("absent", "Absent"),
                    ("present", "Present"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Ulceration, LEFT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="ulceration_right_foot",
            field=models.CharField(
                choices=[
                    ("absent", "Absent"),
                    ("present", "Present"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Ulceration, RIGHT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="vibration_perception_left_toe",
            field=models.CharField(
                choices=[
                    ("present", "Present"),
                    ("decreased", "Decreased"),
                    ("absent", "Absent"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Vibration perception at great toe, LEFT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="vibration_perception_right_toe",
            field=models.CharField(
                choices=[
                    ("present", "Present"),
                    ("decreased", "Decreased"),
                    ("absent", "Absent"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Vibration perception at great toe, RIGHT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="ankle_reflexes_left_foot",
            field=models.CharField(
                choices=[
                    ("present", "Present"),
                    ("present_with_reinforcement", "Present/Reinforcement"),
                    ("absent", "Absent"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Ankle reflexes, LEFT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="ankle_reflexes_right_foot",
            field=models.CharField(
                choices=[
                    ("present", "Present"),
                    ("present_with_reinforcement", "Present/Reinforcement"),
                    ("absent", "Absent"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Ankle reflexes, RIGHT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="examined_left_foot",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If the MNSI assessment was not performed, respond with `not applicable`.",
                max_length=15,
                verbose_name="Was the LEFT foot examined?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="examined_right_foot",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If the MNSI assessment was not performed, respond with `not applicable`.",
                max_length=15,
                verbose_name="Was the RIGHT foot examined?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="monofilament_left_foot",
            field=models.CharField(
                choices=[
                    ("NORMAL", "Normal"),
                    ("reduced", "Reduced"),
                    ("absent", "Absent"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Monofilament, LEFT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="monofilament_right_foot",
            field=models.CharField(
                choices=[
                    ("NORMAL", "Normal"),
                    ("reduced", "Reduced"),
                    ("absent", "Absent"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Monofilament, RIGHT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="normal_appearance_left_foot",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=15,
                verbose_name="Does LEFT foot appear normal?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="normal_appearance_right_foot",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=15,
                verbose_name="Does RIGHT foot appear normal?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="ulceration_left_foot",
            field=models.CharField(
                choices=[
                    ("absent", "Absent"),
                    ("present", "Present"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Ulceration, LEFT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="ulceration_right_foot",
            field=models.CharField(
                choices=[
                    ("absent", "Absent"),
                    ("present", "Present"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Ulceration, RIGHT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="vibration_perception_left_toe",
            field=models.CharField(
                choices=[
                    ("present", "Present"),
                    ("decreased", "Decreased"),
                    ("absent", "Absent"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Vibration perception at great toe, LEFT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="vibration_perception_right_toe",
            field=models.CharField(
                choices=[
                    ("present", "Present"),
                    ("decreased", "Decreased"),
                    ("absent", "Absent"),
                    ("not_examined", "Not examined"),
                ],
                default="not_examined",
                help_text="If the MNSI assessment was not performed or this foot was not examined, respond with `not examined`.",
                max_length=35,
                verbose_name="Vibration perception at great toe, RIGHT foot?",
            ),
        ),
    ]



# Generated by Django 4.2.11 on 2024-06-18 03:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0003_auto_20240618_0505"),
    ]

    operations = [
        migrations.AlterModelTable(
            name="patienthistorymissingbaselinecd4",
            table="patient_history_missing_baseline_cd4_view",
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-16 01:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0065_auto_20240516_0352"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="repeat_fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="repeat_fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="repeat_fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="repeat_fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="repeat_fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
    ]



from datetime import datetime
from typing import Any

from dateutil.relativedelta import MO, relativedelta
from edc_appointment.constants import NEW_APPT
from edc_call_manager.decorators import register
from edc_call_manager.model_caller import ModelCaller
from edc_utils import get_utcnow


@register("edc_appointment.appointment", "meta_subject.subjectvisit")
class ApppointmentReminderModelCaller(ModelCaller):
    verbose_name = "Appointment reminders"
    label = "AppointmentModelCaller"
    app_label = "edc_call_manager"
    consent_model = "meta_consent.subjectconsent"
    locator_model = "edc_locator.subjectlocator"
    subject_model = "edc_registration.registeredsubject"

    @property
    def start_model_options(self) -> dict:
        """Filter for appts due next week"""
        morning = get_utcnow().replace(second=0, hour=0, minute=0)
        monday = morning + relativedelta(weekday=MO(-1))
        return {
            "appt_status": NEW_APPT,
            **{
                "appt_datetime__gte": monday + relativedelta(weeks=1),
                "appt_datetime__lt": monday + relativedelta(weeks=2),
            },
        }

    def call_datetime(self, start_model_obj: Any) -> datetime:
        """Schedule all calls for next week monday"""
        return start_model_obj.appt_datetime - relativedelta(weekday=MO)



# Generated by Django 2.2.6 on 2019-11-07 01:42

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("meta_screening", "0012_auto_20191107_0427")]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose levels",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Blood glucose level 2-hours after glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose levels",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Blood glucose level 2-hours after glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose levels",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Blood glucose level 2-hours after glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose levels",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Blood glucose level 2-hours after glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fasting_glucose",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Fasting glucose levels",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_two_hr",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Blood glucose level 2-hours after glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose)",
            ),
        ),
    ]
